#ifndef _SPIRIT_H
#define _SPIRIT_H

/****************************************************
*
* Spirit OCX control constants
* created by David Wendt on 2/15/99
*
*
*****************************************************/

/*---------------------------------------------------*
 * Sources                                           *
 *---------------------------------------------------*/
#define SRC_VARIABLE            0
#define SRC_TIMER               1
#define SRC_CONSTANT            2
#define SRC_MOTOR_STATUS        3
#define SRC_RANDOM              4
#define SRC_TACHO_COUNTS        5     /* CyberMaster */
#define SRC_TACHO_SPEED         6     /* CyberMaster */
#define SRC_MOTOR_CURRENT       7     /* CyberMaster */
#define SRC_PROGRAM_NUMBER      8     /* RCX */
#define SRC_SENSOR_VALUE        9
#define SRC_SENSOR_TYPE        10
#define SRC_SENSOR_MODE        11
#define SRC_SENSOR_RAW         12     /* RCX */
#define SRC_SENSOR_BOOLEAN     13     /* RCX */
#define SRC_WATCH              14     /* RCX */
#define SRC_PB_MESSAGE         15     /* RCX */
#define SRC_AGC                16     /* CyberMaster */

/*---------------------------------------------------*
 * Timers                                            *
 *---------------------------------------------------*/
#define TIMER_1                 0
#define TIMER_2                 1
#define TIMER_3                 2
#define TIMER_4                 3

/*---------------------------------------------------*
 * Motor Status (macros)                             *
 *---------------------------------------------------*/
#define MS_IS_ON(x)           ((x & 128)==128)
#define MS_IS_BRAKE(x)        ((x & 64)==64)
#define MS_OUTPUT_NUMBER(x)   ((x & 48)>>4)
#define MS_IS_CW(x)           ((x & 4)==4)
#define MS_POWER_LEVEL(x)     (x & 7)

/*---------------------------------------------------*
 * Tachos                                            *
 *---------------------------------------------------*/
#define TACHO_LEFT              0
#define TACHO_RIGHT             1

/*---------------------------------------------------*
 * Program numbers                                   *
 *---------------------------------------------------*/
#define PROGRAM_1               0
#define PROGRAM_2               1
#define PROGRAM_3               2
#define PROGRAM_4               3
#define PROGRAM_5               4

/*---------------------------------------------------*
 * Sensors                                           *
 *---------------------------------------------------*/
#define SENSOR_1                0
#define SENSOR_2                1
#define SENSOR_3                2

/*---------------------------------------------------*
 * Sensor types                                      *
 *---------------------------------------------------*/
#define SENSOR_TYPE_NONE        0
#define SENSOR_TYPE_SWITCH      1
#define SENSOR_TYPE_TEMP        2
#define SENSOR_TYPE_LIGHT       3
#define SENSOR_TYPE_ANGLE       4

/*---------------------------------------------------*
 * Sensor modes                                      *
 *---------------------------------------------------*/
#define SENSOR_MODE_RAW                  0
#define SENSOR_MODE_BOOLEAN              1
#define SENSOR_MODE_TRANSITION_COUNTER   2
#define SENSOR_MODE_PERIODIC_COUNTER     3
#define SENSOR_MODE_PERCENT              4
#define SENSOR_MODE_CELSIUS              5
#define SENSOR_MODE_FAHRENHEIT           6
#define SENSOR_MODE_ANGLE                7

/*---------------------------------------------------*
 * Watch macros                                      *
 *---------------------------------------------------*/
#define WATCH_NUMBER                     0
#define CLOCK_VALUE(h,m)        ((h<<8)+m)

/*---------------------------------------------------*
 * Message                                           *
 *---------------------------------------------------*/
#define PB_MESSAGE_NUMBER                0

/*---------------------------------------------------*
 * Outputs                                           *
 *---------------------------------------------------*/
#define OUTPUT_A                  0
#define OUTPUT_B                  1
#define OUTPUT_C                  2

/*---------------------------------------------------*
 * Operators                                         *
 *---------------------------------------------------*/
#define OP_GT                     0
#define OP_LT                     1
#define OP_EQUAL                  2
#define OP_NOT_EQUAL              3

/*---------------------------------------------------*
 * System sounds                                     *
 *---------------------------------------------------*/
#define SOUND_CLICK               0
#define SOUND_BEEP                1
#define SOUND_SWEEP_DOWN          2
#define SOUND_SWEEP_UP            3
#define SOUND_ERROR               4
#define SOUND_SWEEP_FAST          5

/*---------------------------------------------------*
 * Variables                                         *
 *---------------------------------------------------*/
#define VAR_0                      0
#define VAR_1                      1
#define VAR_2                      2
#define VAR_3                      3
#define VAR_4                      4
#define VAR_5                      5
#define VAR_6                      6
#define VAR_7                      7
#define VAR_8                      8
#define VAR_9                      9
#define VAR_10                    10
#define VAR_11                    11
#define VAR_12                    12
#define VAR_13                    13
#define VAR_14                    14
#define VAR_15                    15
#define VAR_16                    16
#define VAR_17                    17
#define VAR_18                    18
#define VAR_19                    19
#define VAR_20                    20
#define VAR_21                    21
#define VAR_22                    22
#define VAR_23                    23
#define VAR_24                    24
#define VAR_25                    25
#define VAR_26                    26
#define VAR_27                    27
#define VAR_28                    28
#define VAR_29                    29
#define VAR_30                    30
#define VAR_31                    31

/*---------------------------------------------------*
 * Tasks (4 for CyberMaster, 10 for RCX)             *
 *---------------------------------------------------*/
#define TASK_0                     0
#define TASK_1                     1
#define TASK_2                     2
#define TASK_3                     3
#define TASK_4                     4
#define TASK_5                     5
#define TASK_6                     6
#define TASK_7                     7
#define TASK_8                     8
#define TASK_9                     9

/*---------------------------------------------------*
 * Subroutines (4 for CyberMaster, 8 for RCX)        *
 *---------------------------------------------------*/
#define SUB_0                      0
#define SUB_1                      1
#define SUB_2                      2
#define SUB_3                      3
#define SUB_4                      4
#define SUB_5                      5
#define SUB_6                      6
#define SUB_7                      7

/*---------------------------------------------------*
 * Miscellaneous                                     *
 *---------------------------------------------------*/
#define POWER_FULL                 7
#define INFINITE_LOOP              0

/*---------------------------------------------------*
 * Output combinations                               *
 *---------------------------------------------------*/
#define MOTOR_A                    "motor0"
#define MOTOR_C                    "motor2"
#define MOTOR_AC                   "motor0motor2"
#define MOTOR_B                    "motor1"
#define MOTOR_AB                   "motor0motor1"
#define MOTOR_BC                   "motor1motor2"
#define MOTOR_ABC                  "motor0motor1motor2"

/*---------------------------------------------------*
 * Messages                                          *
 *---------------------------------------------------*/
#define SPIRIT_ERR_UNKNOWN               2000
#define SPIRIT_ERR_FATAL                 2001
#define SPIRIT_ERR_PROGRAM_ERROR         2002
#define SPIRIT_ERR_P2T_ENDER             2003
#define SPIRIT_ERR_P2T_ECEIVER           2004
#define SPIRIT_ERR_TOO_MANY_SENDS        2005
#define SPIRIT_ERR_ENDIF_MISSING         2006
#define SPIRIT_ERR_ENDWHILE_MISSING      2007
#define SPIRIT_ERR_ENDLOOP_MISSING       2008
#define SPIRIT_ERR_END_TOO_SOON          2009
#define SPIRIT_ERR_TASK_NESTED_LOOPS     2010
#define SPIRIT_ERR_SUB_NESTED_LOOPS      2011
#define SPIRIT_ERR_ENDOFTASK_EXPECTED    2012
#define SPIRIT_ERR_ENDOFSUB_EXPECTED     2013
#define SPIRIT_ERR_GOSUB_NOT_ALLOWED     2014
#define SPIRIT_ERR_DOWNLOAD              2015
#define SPIRIT_ERR_DOWNLOAD_FIRMWARE     2016
#define SPIRIT_ERR_DOWNLOAD_FROM_FILE    2017
#define SPIRIT_ERR_NOT_ENOUGH_MEMORY     2018
#define SPIRIT_ERR_DOWNLOAD_CHECKSUM     2019
#define SPIRIT_ERR_RAM_CHECKSUM          2020
#define SPIRIT_ERR_RETURN_FROM_BRICK     2021
#define SPIRIT_ERR_RANGE_CHECK           2022
#define SPIRIT_ERR_INVALID_IF_ARGS       2023
#define SPIRIT_ERR_INVALID_WHILE_ARGS    2024
#define SPIRIT_ERR_INVALID_LOOP_ARGS     2025
#define SPIRIT_ERR_DOWNLOAD_UNKNOWN      2026
#define SPIRIT_ERR_ALREADY_IN_DOWNLOAD   2027
#define SPIRIT_ERR_NOT_IN_DOWNLOAD       2028
#define SPIRIT_ERR_BLOCK_SYNTAX_ERROR    2029

#endif

